#include "TextRender.h"


/// Class TextRender > > >
#define TXT_ERROR "Un problme est survenu dans la classe TextRender dveloppe par Jovian. Type : "

TextRender::TextRender( std::string folder, Uint16 nbLiteTxts, Uint16 fontSize )
:m_police( 0x0 ), m_liteTabSize( nbLiteTxts )
{
    /// Initialisation de la TTF
    if ( TTF_Init() < 0 ) std::cout << TXT_ERROR << "initialisation TTF." << std::endl;

    /// Chargement de la police
    m_police = TTF_OpenFont( (folder+"droid.ttf").c_str(), fontSize );
    if ( !m_police ) std::cout << TXT_ERROR << "chargement police." << std::endl;

    /// Dclaration des LiteTextes
    if ( nbLiteTxts != 0 ) {
        m_liteTab = new LiteText*[ nbLiteTxts ];
        for ( Uint16 i(0); i<nbLiteTxts; i++ ) {
            m_liteTab[i] = new LiteText( m_police, "LiteTxt" );
        }
    }
}

TextRender::~TextRender()
{
    /// Destruction des surfaces LiteTxt
    if ( m_liteTabSize != 0 ) {
        for ( Uint16 i(0); i<m_liteTabSize; i++ ) {
            delete m_liteTab[i];
        }
        delete[] m_liteTab;
    }

    /// Destruction des surfaces Name
    for( m_it = m_name.begin(); m_it != m_name.end(); m_it++ )
    {
        SDL_FreeSurface(m_it->second);
        m_it->second = 0x0;
    }

    /// Fermeture TTF
    TTF_CloseFont( m_police );
    TTF_Quit();
}

SDL_Surface* TextRender::takeName( std::string nom )
{
    // Cherche la surface dans le tableau
    m_it = m_name.find(nom);

    // Si la surface est dj charge, on la donne
    if (m_it != m_name.end())
        return m_it->second;

    // Sinon on la charge
    else
        return brutLoadName( nom );
}

SDL_Surface* TextRender::preLoadName( std::string nom, Uint8 r, Uint8 g, Uint8 b )
{
   // Cherche la surface dans le tableau
    m_it = m_name.find(nom);

    // Si la surface est dj charge, on la dtruit
    if (m_it != m_name.end())
        SDL_FreeSurface( m_it->second );

    // On la charge et on la donne
    return brutLoadName( nom , r, g, b );
}

SDL_Surface* TextRender::brutLoadName( std::string nom, Uint8 r, Uint8 g, Uint8 b )
{
    SDL_Color color = {r, g, b};
    SDL_Color background = {255, 255, 255};
    m_name[nom] = TTF_RenderText_Shaded( m_police, nom.c_str(), color, background );
    if (m_name[nom] == 0)
    {
        m_name[nom] = SDL_CreateRGBSurface(SDL_HWSURFACE, 40, 40, 32, 0, 0, 0, 0);
        std::cout << TXT_ERROR << "gnration de texte." << std::endl;
    }

    return m_name[nom];
}

SDL_Surface* TextRender::takeLite( Uint16 ID )
{
    /// Scurit
    if ( ID >= m_liteTabSize ) return 0x0;

    /// Renvoi
    return m_liteTab[ID]->get();
}

SDL_Surface* TextRender::tRendLite( Uint16 ID, std::string texte, Uint8 r, Uint8 g, Uint8 b )
{
    /// Scurit
    if ( ID >= m_liteTabSize ) return 0x0;

    /// Rendering
    m_liteTab[ID]->render( texte, r, g, b);

    /// Renvoi
    return m_liteTab[ID]->get();
}

LiteText* TextRender::takeLiteObject( Uint16 ID )
{
    /// Scurit
    if ( ID >= m_liteTabSize ) return 0x0;

    /// Renvoi
    return m_liteTab[ID];
}


/// Class LiteText > > >

LiteText::LiteText( TTF_Font* police, std::string text )
:m_text( 0x0 ), m_policeCopy( police )
{
    render( text );
}

LiteText::~LiteText()
{
    if ( m_text != 0x0 )
            SDL_FreeSurface( m_text );
}

void LiteText::render( std::string text, Uint8 r, Uint8 g, Uint8 b )
{
    if ( !text.empty() )
    {
        if ( m_text != 0x0 )
            SDL_FreeSurface( m_text );

        /// Rendering
        SDL_Color color = { r, g, b };
        m_text = TTF_RenderText_Solid( m_policeCopy, text.c_str(), color );
    }
}

SDL_Surface* LiteText::get()
{
    return m_text;
}
